/************************************************************************/
/* ôҪu-bootֲġ                                     */
/************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"
#include "cmd.h"

extern cmd_tbl_s __user_cmd_start;
extern cmd_tbl_s __user_cmd_end;

static void			__cmd_process_macros(const char* src, char* dst);
static int			__cmd_parse_line(char* line, char* argv[]);

static void __cmd_process_macros(const char* src, char* dst)
{
	char c;
	char prev;
	const char* var_name_start = NULL;
	int src_cnt = strlen (src);
	int dst_cnt = CONFIG_USER_CMD_SIZE;
	int state = 0;		/* 0 = waiting for '$'  */

	/* 1 = waiting for '(' or '{' */
	/* 2 = waiting for ')' or '}' */
	/* 3 = waiting for '''  */

	prev = '\0';		/* previous character   */

	while (src_cnt && dst_cnt)
	{
		c = *src++;
		src_cnt--;

		if (state != 3)
		{
			/* remove one level of escape characters */
			if ((c == '\\') && (prev != '\\'))
			{
				if (src_cnt-- == 0)
				{
					break;
				}
				prev = c;
				c = *src++;
			}
		}

		switch (state)
		{
			case 0:	/* Waiting for (unescaped) $    */
				if ((c == '\'') && (prev != '\\'))
				{
					state = 3;
					break;
				}
				if ((c == '$') && (prev != '\\'))
				{
					state++;
				}
				else
				{
					*(dst++) = c;
					dst_cnt--;
				}
				break;

			case 1:	/* Waiting for (        */
				if (c == '(' || c == '{')
				{
					state++;
					var_name_start = src;
				}
				else
				{
					state = 0;
					*(dst++) = '$';
					dst_cnt--;

					if (dst_cnt) 
					{
						*(dst++) = c;
						dst_cnt--;
					}
				}
				break;

			case 2:	/* Waiting for )        */
				if (c == ')' || c == '}') 
				{
					int i;
					char envname[CONFIG_USER_CMD_SIZE], *envval;
					int envcnt = src - var_name_start - 1;	/* Varname # of chars */

					/* Get the varname */
					for (i = 0; i < envcnt; i++)
					{
						envname[i] = var_name_start[i];
					}
					envname[i] = 0;

					/* Get its value */
					envval = getenv(envname);

					/* Copy into the line if it exists */
					if (envval != NULL)
					{
						while ((*envval) && dst_cnt)
						{
							*(dst++) = *(envval++);
							dst_cnt--;
						}
						/* Look for another '$' */
						state = 0;
					}
				}
				break;

			case 3:	/* Waiting for '        */
				if ((c == '\'') && (prev != '\\')) 
				{
					state = 0;
				}
				else 
				{
					*(dst++) = c;
					dst_cnt--;
				}
				break;

			default:
				break;
		}
		prev = c;
	}

	if (dst_cnt)
	{
		*dst = 0;
	}
	else
	{
		*(dst - 1) = 0;
	}
}

static int __cmd_parse_line(char* line, char* argv[])
{
	int arg_cnt = 0;

	while (arg_cnt < CONFIG_USER_CMD_MAX_ARG_CNT)
	{
		/* skip any white space */
		while ((*line == ' ') || (*line == '\t'))
		{
			++line;
		}

		if (*line == '\0')
		{	
			/* end of line, no more args	*/
			argv[arg_cnt] = NULL;
			return arg_cnt;
		}

		argv[arg_cnt++] = line;	/* begin of argument string	*/

		/* find end of string */
		while (*line && (*line != ' ') && (*line != '\t'))
		{
			++line;
		}

		if (*line == '\0')
		{	
			/* end of line, no more args */
			argv[arg_cnt] = NULL;
			return arg_cnt;
		}

		*line++ = '\0';		/* terminate current arg */
	}

	printf ("** Too many arg count (max. %d) **\n", CONFIG_USER_CMD_MAX_ARG_CNT);

	return arg_cnt;
}

cmd_tbl_s* cmd_find_from_table(const char* cmd, cmd_tbl_s* table, int table_len)
{
	cmd_tbl_s* cmdtp;
	cmd_tbl_s* cmdtp_temp = table;
	const char* p;
	int len;
	int found = 0;

	if (!cmd)
		return NULL;
	/*
	* Some commands allow length modifiers (like "cp.b");
	* compare command name only until first dot.
	*/
	len = ((p = strchr(cmd, '.')) == NULL) ? strlen (cmd) : (p - cmd);

	for (cmdtp = table; cmdtp != table + table_len; cmdtp++)
	{
		if (strncmp (cmd, cmdtp->name, len) == 0)
		{
			//ȫƥ
			if (len == strlen (cmdtp->name))
			{
				return cmdtp;
			}

			/*д*/
			cmdtp_temp = cmdtp;
			found++;
		}
	}

	//ҵΨһ
	if (found == 1)
	{
		return cmdtp_temp;
	}

	//ûлʹдʱڶ
	return NULL;
}

cmd_tbl_s* cmd_find(const char* cmd)
{
	int len = &__user_cmd_end - &__user_cmd_start;
	return cmd_find_from_table(cmd, &__user_cmd_start, len);
}

int cmd_main_loop(void)
{
	char* cmd;
	
	while(1)
	{
#ifdef CONFIG_USER_CMD_PROMPT
		printf("%s", CONFIG_USER_CMD_PROMPT);
#endif
		cmd = readline(NULL);
		cmd_run(cmd, 0);
		free(cmd);
	}

	return 0;
}

int cmd_print_usage(cmd_tbl_s *cmdtp)
{
	printf("%s - %s\n\n", cmdtp->name, cmdtp->usage);
	printf("Usage:\n%s ", cmdtp->name);
	if (!cmdtp->help) {
		puts ("- No additional help available.\n");
		return 1;
	}
	puts (cmdtp->help);
	putchar('\n');

	return 1;
}

int cmd_run(const char* cmd, int flag)
{
	cmd_tbl_s* cmdtp;
	char cmd_buf[CONFIG_USER_CMD_SIZE];				/* working copy of cmd		*/
	char* token;									/* start of token in cmdbuf	*/
	char* sep;										/* end of token (separator) in cmdbuf */
	char final_token[CONFIG_USER_CMD_SIZE];
	char* str = cmd_buf;
	char* argv[CONFIG_USER_CMD_MAX_ARG_CNT + 1];	/* NULL terminated	*/
	int argc;
	int inquotes;
	int repeatable = 1;
	int rc = 0;

	if (!cmd || !*cmd) 
	{
		return -1;
	}

	if (strlen(cmd) >= CONFIG_USER_CMD_SIZE)
	{
		printf("## Command too long!\n");
		return -1;
	}

	strcpy (cmd_buf, cmd);

	/* Process separators and check for invalid
	* repeatable commands
	*/
	while (*str)
	{
		/*
		* Find separator, or string end
		* Allow simple escape of ';' by writing "\;"
		*/
		for (inquotes = 0, sep = str; *sep; sep++)
		{
			if(*sep == '\'' && *(sep-1) != '\\')
			{
				inquotes = !inquotes;
			}

			if(!inquotes &&
				(*sep == ';') &&	/* separator		*/
				( sep != str) &&	/* past string start	*/
				(*(sep-1) != '\\'))	/* and NOT escaped	*/
			{
				break;
			}
		}

		/*
		* Limit the token to data between separators
		*/
		token = str;
		if (*sep)
		{
			str = sep + 1;	/* start of command for next pass */
			*sep = '\0';
		}
		else
		{
			str = sep;	/* no more commands for next pass */
		}

		/* find macros in this token and replace them */
		__cmd_process_macros(token, final_token);

		/* Extract arguments */
		if ((argc = __cmd_parse_line (final_token, argv)) == 0) {
			rc = -1;	/* no command at all */
			continue;
		}

		/* Look up command in command table */
		if ((cmdtp = cmd_find(argv[0])) == NULL)
		{
			printf ("Unknown command '%s' - try 'help'\n", argv[0]);
			rc = -1;	/* give up after bad command */
			continue;
		}

		/* found - check max args */
		if (argc > cmdtp->max_arg_cnt)
		{
			cmd_print_usage(cmdtp);
			rc = -1;
			continue;
		}

		/* OK - call function to do the command */
		if ((cmdtp->do_cmd) (cmdtp, flag, argc, argv) != 0)
		{
			rc = -1;
		}

		repeatable &= cmdtp->repeatable;
	}

	return rc ? rc : repeatable;
}

char* readline(const char* promt)
{
	char* buf;
	char ch;
	int i = 0;

	buf = malloc(1024);

	memset(buf, 0, 1024);
	while((ch = getchar()) != '\n')
	{
		if(i < 1024 - 1)
		{
			buf[i] = ch;
		}
		i++;
	}

	return buf;
}