#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <test/test_func.h>
#include "cmd.h"
#include "config.h"

static int cmd_get_unit_by_name(const char* name)
{
	struct __name_type
	{
		const char* name;
		int type;
	};

	struct __name_type name_type[4] = 
	{
		{"func",		TEST_UNIT_FUNC},
		{"module",		TEST_UNIT_MODULE},
		{"file",		TEST_UNIT_FILE},
		{"lib",			TEST_UNIT_LIB},
	};
	int i;
	int cnt = sizeof(name_type) / sizeof(struct __name_type);

	for(i = 0; i < cnt; i++)
	{
		if(strcmp(name_type[i].name, name) == 0)
		{
			return name_type[i].type;
		}
	}

	return TEST_UNIT_INVALID;
}

static int cmd_get_mode_by_name(const char* name)
{
	struct __name_type
	{
		const char* name;
		int type;
	};

	struct __name_type name_type[2] = 
	{
		{"auto",		TEST_MODE_TEST_AUTO},
		{"user",		TEST_MODE_TEST_USER},
	};
	int i;
	int cnt = sizeof(name_type) / sizeof(struct __name_type);

	for(i = 0; i < cnt; i++)
	{
		if(strcmp(name_type[i].name, name) == 0)
		{
			return name_type[i].type;
		}
	}

	return TEST_MODE_TEST_INVALID;
}

static char* cmd_get_unit_name(char* name, int type)
{
	switch(type)
	{
		case TEST_UNIT_FUNC:
			strcpy(name, "func");
			break;

		case TEST_UNIT_MODULE:
			strcpy(name, "module");
			break;

		case TEST_UNIT_FILE:
			strcpy(name, "file");
			break;

		case TEST_UNIT_LIB:
			strcpy(name, "lib");
			break;

		default:
			strcpy(name, "");
			break;
	}

	return name;
}

static char* cmd_get_mode_name(char* name, int type)
{
	switch(type)
	{
	case TEST_MODE_TEST_AUTO:
		strcpy(name, "auto");
		break;

	case TEST_MODE_TEST_USER:
		strcpy(name, "user");
		break;

	default:
		strcpy(name, "");
		break;
	}

	return name;
}

static int cmd_do_test_print_env(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	int value;
	char env_value[1024];
	const char* env;

	//ӡл
	if(argc == 1)
	{
		value = test_get_unit();
		cmd_get_unit_name(env_value, value);
		printf("unit = %s\n", env_value);

		value = test_get_test_mode();
		cmd_get_mode_name(env_value, value);
		printf("mode = %s\n", env_value);

		return 0;
	}

	env = argv[1];

	if(strcmp(env, "unit") == 0)
	{
		value = test_get_unit();
		cmd_get_unit_name(env_value, value);
		printf("unit = %s\n", env_value);

		return 0;
	}

	if(strcmp(env, "mode") == 0)
	{
		value = test_get_test_mode();
		cmd_get_mode_name(env_value, value);
		printf("mode = %s\n", env_value);

		return 0;
	}

	printf("The \'%s\' is invalid env.\n", env);

	return 0;
}

static int cmd_do_test_run_menu(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	int index;
	test_item_s* item;

	index = atoi(argv[1]);

	item = test_find_item(index);
	if(!item)
	{
		printf("Invalid menu index!Please input a valid index.\n");
		return -1;
	}
	test_run_item(item);

	return 0;
}

static int cmd_do_test_show_cur_menu(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	test_show_menu();

	return 0;
}

static int cmd_do_test_show_up_menu(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	test_menu_up();
	test_show_menu();

	return 0;
}

static int cmd_do_test_sel_menu(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	int index;

	index = atoi(argv[1]);

	if(test_sel_item(index) < 0)
	{
		printf("Invalid menu index!Please input a valid index.\n");
		return -1;
	}

	return 0;
}

static int cmd_do_test_set_env(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	int i;
	int value;

	for(i = 1; i < argc; i += 2)
	{
		if(strcmp(argv[i], "-unit") == 0 ||  strcmp(argv[i], "unit") == 0)
		{
			value = cmd_get_unit_by_name(argv[i + 1]);
			if(value == TEST_UNIT_INVALID)
			{
				printf("The \'%s\' is invalid unit value.\n", argv[i + 1]);
				continue;
			}
			
			test_set_unit(value);
			printf("Set \'%s\' successfully.\n", argv[i]);
			continue;
		}

		if(strcmp(argv[i], "-mode") == 0 ||  strcmp(argv[i], "mode") == 0)
		{
			value = cmd_get_mode_by_name(argv[i + 1]);
			if(value == TEST_MODE_TEST_INVALID)
			{
				printf("The \'%s\' is invalid mode value.\n", argv[i + 1]);
				continue;
			}

			test_set_test_mode(value);
			printf("Set \'%s\' successfully.\n", argv[i]);
			continue;
		}

		printf("The \'%s\' is invalid env.\n", argv[i]);
	}

	return 0;
}

static int cmd_do_test(cmd_tbl_s * cmdtp, int flag, int argc, char * const argv[])
{
	const char* cmd;

	//2
	if (argc < 2)
	{
		goto GOTOPRINTUSAGE;
	}

	cmd = argv[1];

	if(strcmp(cmd, "menu") == 0)
	{
		return cmd_do_test_show_cur_menu(cmdtp, flag, argc - 1, argv + 1);
	}

	if(strcmp(cmd, "menuup") == 0)
	{
		return cmd_do_test_show_up_menu(cmdtp, flag, argc - 1, argv + 1);
	}

	if(strcmp(cmd, "print") == 0)
	{
		return cmd_do_test_print_env(cmdtp, flag, argc - 1, argv + 1);
	}

	if(strcmp(cmd, "run") == 0)
	{
		if(argc < 3)
		{
			goto GOTOPRINTUSAGE;
		}
		return cmd_do_test_run_menu(cmdtp, flag, argc - 1, argv + 1);
	}

	if(strcmp(cmd, "sel") == 0)
	{
		if(argc < 3)
		{
			goto GOTOPRINTUSAGE;
		}
		return cmd_do_test_sel_menu(cmdtp, flag, argc - 1, argv + 1);
	}

	if(strcmp(cmd, "set") == 0)
	{
		if(argc < 4)
		{
			goto GOTOPRINTUSAGE;
		}
		return cmd_do_test_set_env(cmdtp, flag, argc - 1, argv + 1);
	}

GOTOPRINTUSAGE:
	return cmd_print_usage(cmdtp);
}

USER_CMD(
		test, CONFIG_USER_CMD_MAX_ARG_CNT, 1, cmd_do_test,
		"test libarary command.",
		"test menu - show current menu.\n"
		"test menuup - goto the up menu and show the menu.\n"
		"test print -env value ... - print the test environment value.\n"
		"		if the env is none, it will print all env values.\n"
		"test run menu_num args - test the selected menu.\n"
		"test sel menu_num args - goto the selected menu.\n"
		"		the menu_num is menu number, just like 1, 2, 3.\n"
		"		if the args is none, the args will use default args.(Don't support)\n"
		"test set -env value ... - set the test environment value.\n"
		"		environment include the following:\n"
		"		-unit: the test min unit. The value could be func, module, file, lib.\n"
		"		-mode: the test mode. The value could be auto, user.\n"
		);