#include <string.h>
#include <stdlib.h>
#include "array.h"

void arr_init(array_s* arr)
{
	if(!arr)
	{
		return;
	}

	memset(arr, 0, sizeof(array_s));
}

bool arr_create(array_s* arr, void* obj_arr, int obj_size, int count, int mode)
{
	uchar* new_obj_arr = NULL;

	if(!arr || obj_size <= 0 || count < 0)
	{
		return FALSE;
	}

	//ûָ
	if(mode & ARR_MODE_CREATE_USER)
	{
		if(!obj_arr)
		{
			return FALSE;
		}

		new_obj_arr = obj_arr;
		arr->__state |= ARR_STATE_USER_CREATE;
	}
	else
	{
		if(count > 0)
		{
			new_obj_arr = (uchar*)malloc(obj_size * count);
		}
	}

	arr->__obj_arr = new_obj_arr;
	arr->__obj_size = obj_size;
	arr->__count = count;

	return TRUE;
}

void arr_destroy(array_s* arr)
{
	if(arr_is_empty(arr))
	{
		return;
	}

	//ϵͳԶģͷڴ
	if(!(arr->__state & ARR_STATE_USER_CREATE))
	{
		if(arr->__obj_arr)
		{
			free(arr->__obj_arr);
			arr->__obj_arr = NULL;
		}
	}

	memset(arr, 0, sizeof(array_s));
}

int arr_add(array_s* arr, const void* src_obj, int count)
{
	uchar* new_obj_arr;
	int ret;

	if(!arr_is_valid(arr) || !src_obj || count <= 0)
	{
		return -1;
	}

	//ûָ룬
	if(arr->__state & ARR_STATE_USER_CREATE)
	{
		return -1;
	}

	//ڴ
	new_obj_arr = malloc(arr->__obj_size * (arr->__count + count));

	//ԭ
	memcpy(new_obj_arr, arr->__obj_arr, arr->__obj_size * arr->__count);
	//
	memcpy(new_obj_arr + arr->__obj_size * arr->__count, src_obj, 
		arr->__obj_size * count);
	//ͷԭڴ
	free(arr->__obj_arr);

	ret = arr->__count;

	arr->__obj_arr = new_obj_arr;
	arr->__count += count;

	return ret;
}

int arr_insert(array_s* arr, const void* src_obj, int count, int index)
{
	uchar* new_obj_arr;

	if(arr_is_empty(arr) || !src_obj || count <= 0 || index < 0)
	{
		return -1;
	}

	//Ѿɶ
	if(index >= arr->__count)
	{
		return -1;
	}

	//ûָ룬ܲ
	if(arr->__state & ARR_STATE_USER_CREATE)
	{
		return -1;
	}

	//ڴ
	new_obj_arr = malloc(arr->__obj_size * (arr->__count + count));
	if(index > 0)
	{
		//ǰĶ
		memcpy(new_obj_arr, arr->__obj_arr, arr->__obj_size * index);
	}
	//
	memcpy(new_obj_arr + arr->__obj_size * index, src_obj, 
		arr->__obj_size * count);
	//Ķ
	memcpy(new_obj_arr + arr->__obj_size * (index + count), 
		(uchar*)arr->__obj_arr + arr->__obj_size * index, 
		arr->__obj_size * (arr->__count - index));
	//ͷԭڴ
	free(arr->__obj_arr);

	arr->__obj_arr = new_obj_arr;
	arr->__count += count;

	return index;
}

void arr_remove(array_s* arr, int index, int count)
{
	uchar* new_obj_arr;
	int real_count;

	if(arr_is_empty(arr) || count <= 0 || index < 0)
	{
		return;
	}

	//Ѿɶ
	if(index >= arr->__count)
	{
		return;
	}

	//ûָ룬Ƴ
	if(arr->__state & ARR_STATE_USER_CREATE)
	{
		return;
	}

	//ʵʿɾ
	if(index + count > arr->__count)
	{
		real_count = arr->__count - index;
	}
	else
	{
		real_count = count;
	}

	if(arr->__count - real_count > 0)
	{
		//ڴ
		new_obj_arr = malloc(arr->__obj_size * (arr->__count - real_count));

		//ƳĶǰ
		memmove((uchar*)arr->__obj_arr + arr->__obj_size * index,  
			(uchar*)arr->__obj_arr + arr->__obj_size * (index + real_count), 
			arr->__obj_size * (arr->__count - index - real_count));

		memcpy(new_obj_arr, arr->__obj_arr, arr->__obj_size * (arr->__count - real_count));
	}
	else
	{
		//ɾѾĶ
		new_obj_arr = NULL;
	}
	//ͷԭڴ
	free(arr->__obj_arr);

	arr->__obj_arr = new_obj_arr;
	arr->__count -= real_count;
}

void arr_remove_all(array_s* arr)
{
	if(arr_is_empty(arr))
	{
		return;
	}

	//ͷԭڴ
	free(arr->__obj_arr);

	arr->__obj_arr = NULL;
	arr->__count = 0;
}

int arr_get_at(array_s* arr, void* dst_obj, int count, int index)
{
	int real_count;

	if(arr_is_empty(arr) || !dst_obj || count < 0 || index < 0)
	{
		return -1;
	}

	//Ѿɶ
	if(index >= arr->__count)
	{
		return -1;
	}

	//ʵʿɾ
	if(index + count > arr->__count)
	{
		real_count = arr->__count - index;
	}
	else
	{
		real_count = count;
	}

	//ƶ
	memcpy(dst_obj, (uchar*)arr->__obj_arr + arr->__obj_size * index, arr->__obj_size * real_count);

	return real_count;
}

void* arr_get_obj_ptr(array_s* arr, int index)
{
	if(arr_is_empty(arr) || index < 0)
	{
		return NULL;
	}

	//Ѿɶ
	if(index >= arr->__count)
	{
		return NULL;
	}

	return (uchar*)arr->__obj_arr + arr->__obj_size * index;
}

int arr_get_count(array_s* arr)
{
	if(arr_is_empty(arr))
	{
		return 0;
	}

	return arr->__count;
}

bool arr_is_empty(array_s* arr)
{
	if(!arr_is_valid(arr))
	{
		return TRUE;
	}

	if(!arr->__obj_arr || !arr->__count)
	{
		return TRUE;
	}

	return FALSE;
}

bool arr_is_valid(array_s* arr)
{
	if(!arr)
	{
		return FALSE;
	}

	if(!arr->__obj_size)
	{
		return FALSE;
	}

	return TRUE;
}