#include <ctype.h>
#include <string.h>
#include "func.h"

int x_atoh(uchar* dst, const char* src, int len)
{
	int i = 0;
	uchar tmp = 0;
	int odd = 0;

	if(len % 2)
	{
		odd = 1;
	}

	for(i = 0; i < len; i++)
	{
		if(src[i] >= '0' && src[i] <= '9')				//0-9
		{
			tmp = src[i] - '0';
		}
		else if(src[i] >= 'A' && src[i] <= 'F')			//A-F
		{
			tmp = src[i] - 'A' + 10;
		}
		else if(src[i] >= 'a' && src[i] <= 'f')			//a-f
		{
			tmp = src[i] - 'a' + 10;
		}
		else
		{
			return i;
		}

		if((i + odd) % 2)
		{
			dst[(i + odd) / 2] = (dst[(i + odd) / 2] << 4) | tmp;
		}
		else
		{
			dst[(i + odd) / 2] = tmp;
		}
	}

	return (len + odd) / 2;
}

int	x_bcdtoh(uchar* dst, const uchar* src, int len)
{
	int i;

	if(!dst || !src || len < 0)
	{
		return -1;
	}

	for(i = 0; i < len; i++)
	{
		if((*(src + i) & 0xF0) > 9 || (*(src + i) & 0xF) > 9)
		{
			return i;
		}

		*(dst + i) = (*(src + i) & 0xF0) * 10 + (*(src + i) & 0xF);
	}

	return len;
}

char* x_htoa(char* dst, const uchar* src, int len)
{
	const char* digits_tbl = "0123456789ABCDEF";
	int i;

	for(i = 0; i < len; i++)
	{
		dst[2 * i] = digits_tbl[src[i] >> 4];
		dst[2 * i + 1] = digits_tbl[src[i] & 0x0F];
	}

	dst[2 * i] = '\0';

	return dst;
}

int	x_htobcd(uchar* dst, const uchar* src, int len)
{
	int i;

	if(!dst || !src || len < 0)
	{
		return -1;
	}

	for(i = 0; i < len; i++)
	{
		if(*(src + i) > 99)
		{
			return i;
		}

		*(dst + i) = ((*(src + i) / 10) << 4) + *(src + i) % 10;
	}

	return len;
}

char* x_itoa(char* dst, int value, uint radix)
{
	const char* digits_tbl = "0123456789abcdefghijklmnopqrstuvwxyz";
	char* ret = dst;
	char tmp[66];
	int i = 0;

	if(!dst || radix < 2 || radix > 32)
	{
		return NULL;
	}

	if(value < 0)
	{
		value = - value;
		*dst = '-';
		dst++;
	}

	if(value)
	{
		while (value != 0)
		{
			tmp[i++] = digits_tbl[value % radix];
			value /= radix;
		}
	}
	else
	{
		tmp[0] = '0';
		i++;
	}

	while(i-- > 0)
	{
		*(dst++) = tmp[i];
	}

	*dst = '\0';
	
	return ret;
}

int x_get_bit(int src, uint bit, uint radix)
{
	int bit_autho;

	if(radix < 2 || radix > 32)
	{
		return -1;
	}

	if(radix == X_RADIX_2)
	{
		if(src & (0x0001 << bit))
		{
			return 1;
		}
		return 0;
	}

	bit_autho = 1;
	if(bit > 0)
	{
		while(bit--)
		{
			bit_autho *= radix;		//For example, 10x is 10 * 10.
		}
	}

	return src / bit_autho % radix;
}

int x_set_bit(int src, uint value, uint bit, uint radix)
{
	int bit_autho;

	if(radix < 2 || radix > 32 || value >= radix)
	{
		return -1;
	}
	
	if(radix == X_RADIX_2)
	{
		if(value)
		{
			src |= 0x0001 << bit;
		}
		else
		{
			if(src & (0x0001 << bit))
			{
				src &= ~(0x0001 << bit);
			}
		}

		return src;
	}

	bit_autho = 1;
	if(bit > 0)
	{
		while(bit--)
		{
			bit_autho *= radix;		//For example, 10x is 10 * 10.
		}
	}

	src -= (src / bit_autho % radix) * bit_autho;
	src += value * bit_autho;

	return src;
}

