#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include "test_func.h"

static int				test_mode = TEST_MODE_TEST_DEFAULT;
static int				test_unit = TEST_UNIT_DEFAULT;
static test_item_s		top_item = 
{
	.name		= "top item",
	.type		= TEST_TYPE_ITEM_TOP,
	.parent		= NULL,
};
static test_item_s*		cur_item = &top_item;

static		void __test_setup_menu_index(test_item_s* item);

static void __test_setup_menu_index(test_item_s* item)
{
	test_item_s* sub_item;
	int sub_item_cnt;
	int i;
	int index = 0;

	test_get_sub_item(item, &sub_item, &sub_item_cnt);

	if(!sub_item)
	{
		return;
	}

	for(i = 0; i < sub_item_cnt; i++)
	{
		if(0 == strcmp(sub_item[i].name, "") || !sub_item[i].menu.do_menu)
		{
			sub_item[i].menu.index = TEST_INVALID_INDEX;
			continue;
		}

		sub_item[i].menu.index = index;
		__test_setup_menu_index(sub_item + i);
		index++;
	}
}

int	test_add_sub_item(test_item_s* item, test_item_s* sub_item_arr, int sub_item_cnt)
{
	int i;

	for(i = 0; i < sub_item_cnt; i++)
	{
		sub_item_arr[i].parent = item;
	}

	menu_add_item_arr(&item->menu, (menu_s*)sub_item_arr, sub_item_cnt);

	return sub_item_cnt;
}

int test_do_nothing(void* arg)
{
	return PASS;
}

test_item_s* test_find_item(int index)
{
	test_item_s* item;
	int	cnt;
	int i;

	test_get_sub_item(cur_item, &item, &cnt);

	if(!item)
	{
		return NULL;
	}

	for(i = 0; i < cnt; i++)
	{
		if(item[i].menu.index == index)
		{
			return item + i;
		}
	}

	return NULL;
}

void test_get_sub_item(test_item_s* item, test_item_s** sub_item_arr, int* sub_item_cnt)
{
	menu_get_item_arr(&item->menu, (menu_s**)sub_item_arr, sub_item_cnt);
}

int test_get_test_mode(void)
{
	return test_mode;
}

test_item_s* test_get_top_item(void)
{
	return &top_item;
}

char* test_get_type_name(char* name, int type)
{
	switch(type)
	{
		case TEST_TYPE_ITEM_FILE:
			strcpy(name, "file");
			break;

		case TEST_TYPE_ITEM_FUNC:
			strcpy(name, "function");
			break;

		case TEST_TYPE_ITEM_LIB:
			strcpy(name, "lib");
			break;

		case TEST_TYPE_ITEM_MISC:
			strcpy(name, "misc");
			break;

		case TEST_TYPE_ITEM_MODULE:
			strcpy(name, "module");
			break;

		default:
			strcpy(name, "");
			break;
	}

	return name;
}

int	test_get_unit(void)
{
	return test_unit;
}

int test_get_user_answer(void)
{
	enum
	{
		MAX_BUF_COUN	= 10,
	};
	int i = 0;
	char ch;
	char buf[MAX_BUF_COUN];

	memset(buf, 0, MAX_BUF_COUN);
	while((ch = getchar()) != '\n')
	{
		if(i < MAX_BUF_COUN - 1)
		{
			buf[i] = ch;
		}
		i++;
	}

	if(i >= MAX_BUF_COUN)
	{
		return NO;
	}

	if(!strcmp(buf, "y") || !strcmp(buf, "yes") || !strcmp(buf, "Y") || 
		!strcmp(buf, "Yes") || !strcmp(buf, "Y"))
	{
		return YES;
	}

	return NO;
}

void test_init_item(test_item_s* item, const char* name, test_do_f test, int type, 
					const char* menu_name, test_item_s* parent_item)
{
	memset(item, 0, sizeof(test_item_s));

	if(name)
	{
		strcpy(item->name, name);
	}
	item->type = type;

	if(menu_name)
	{
		strcpy(item->menu.name, menu_name);
	}
	item->menu.do_menu = test;

	item->parent = parent_item;
}

void test_init_file(test_item_s* item, const char* name, test_do_f test, 
					const char* menu_name, test_item_s* parent_item)
{
	test_init_item(item, name, test, TEST_TYPE_ITEM_FILE, menu_name, parent_item);
}

void test_init_func(test_item_s* item, const char* name, test_do_f test, 
					const char* menu_name, test_item_s* parent_item)
{
	test_init_item(item, name, test, TEST_TYPE_ITEM_FUNC, menu_name, parent_item);
}

void test_init_lib(test_item_s* item, const char* name, test_do_f test, 
				   const char* menu_name, test_item_s* parent_item)
{
	test_init_item(item, name, test, TEST_TYPE_ITEM_LIB, menu_name, parent_item);
}

void test_init_misc(test_item_s* item, const char* name, test_do_f test, 
					const char* menu_name, test_item_s* parent_item)
{
	test_init_item(item, name, test, TEST_TYPE_ITEM_MISC, menu_name, parent_item);
}

void test_init_module(test_item_s* item, const char* name, test_do_f test, 
					  const char* menu_name, test_item_s* parent_item)
{
	test_init_item(item, name, test, TEST_TYPE_ITEM_MODULE, menu_name, parent_item);
}

bool test_is_in_unit(const test_item_s* item, int unit)
{
	int type = item->type;

	if(!test_is_invalid_type(type))
	{
		return FALSE;
	}

	switch(unit)
	{
		case TEST_UNIT_FUNC:
			if(type == TEST_TYPE_ITEM_FUNC || type == TEST_TYPE_ITEM_MISC)
			{
				return TRUE;
			}
			break;

		case TEST_UNIT_MODULE:
			if(type == TEST_TYPE_ITEM_FUNC || type == TEST_TYPE_ITEM_MISC || 
				type == TEST_TYPE_ITEM_MODULE)
			{
				return TRUE;
			}
			break;

		case TEST_UNIT_FILE:
			if(type == TEST_TYPE_ITEM_FUNC || type == TEST_TYPE_ITEM_MISC || 
				type == TEST_TYPE_ITEM_MODULE || type == TEST_TYPE_ITEM_FILE)
			{
				return TRUE;
			}
			break;

		case TEST_UNIT_LIB:
			if(type == TEST_TYPE_ITEM_FUNC || type == TEST_TYPE_ITEM_MISC || 
				type == TEST_TYPE_ITEM_MODULE || type == TEST_TYPE_ITEM_FILE || 
				type == TEST_TYPE_ITEM_LIB)
			{
				return TRUE;
			}
			break;

		default:
			break;
	}

	return FALSE;
}

bool test_is_invalid_type(int type)
{
	switch(type)
	{
		case TEST_TYPE_ITEM_FUNC:
		case TEST_TYPE_ITEM_MISC:
		case TEST_TYPE_ITEM_MODULE:
		case TEST_TYPE_ITEM_FILE:
		case TEST_TYPE_ITEM_LIB:
			return TRUE;
			break;

		default:
			return FAIL;
			break;
	}

	return TRUE;
}

void test_menu_up(void)
{
	if(cur_item->parent)
	{
		cur_item = cur_item->parent;
	}
}

void test_print_ask_check(const char* format, ...)
{
	va_list  vlist;
	char info[1024];

	va_start(vlist, format);
	vsprintf(info, format, vlist);
	va_end(vlist);

	printf("Please check it! The %s function is OK? yes/no.", info);
}

void test_print_ask_test(const char* name, int type)
{
	char type_name[100];

	if(strcmp(test_get_type_name(type_name, type), "") == 0)
	{
		printf("Warning: Invalid type!\n");
		return;
	}

	printf("Do you want to test %s %s?", name, type_name);
}

void test_print_err(const char* name, const char* format, ...)
{
	va_list  vlist;
	char info[1024];

	va_start(vlist, format);
	vsprintf(info, format, vlist);
	va_end(vlist);

	printf("%s error: %s!\n", name, info);
}

void test_print_fail(const char* name, int type)
{
	char type_name[100];

	if(strcmp(test_get_type_name(type_name, type), "") == 0)
	{
		printf("Warning: Invalid type!\n");
		return;
	}

	printf("The %s %s is failed!\n", type_name, name);
}

void test_print_pass(const char* name, int type)
{
	char type_name[100];

	if(strcmp(test_get_type_name(type_name, type), "") == 0)
	{
		printf("Warning: Invalid type!\n");
		return;
	}

	printf("The %s %s is passed!\n", type_name, name);
}

int test_run_item(test_item_s* item)
{
	test_item_s* sub_item;
	int sub_item_cnt;
	int i;
	int err = 0;

	test_get_sub_item(item, &sub_item, &sub_item_cnt);

	if(!sub_item)
	{
		if(FAIL == item->menu.do_menu(item->menu.arg))
		{
			test_print_fail(item->name, item->type);	
			return 1;
		}

		test_print_pass(item->name, item->type);
		return 0;
	}

	for(i = 0; i < sub_item_cnt; i++)
	{
		err = test_run_item(sub_item + i);
		if(err)
		{
			test_print_fail(item->name, item->type);
			return err;
		}
	}

	test_print_pass(item->name, item->type);
	
	return err;
}

void test_set_test_mode(int mode)
{
	switch(mode)
	{
		case TEST_MODE_TEST_AUTO:
		case TEST_MODE_TEST_USER:
			test_mode = mode;
			break;

		default:
			printf("Warn: Invalid test mode!\n");
			break;
	}
}

int test_sel_item(int index)
{
	test_item_s* item;

	item = test_find_item(index);
	if(!item)
	{
		return -1;
	}

	if(!test_is_in_unit(item, test_unit))
	{
		cur_item = item;
		test_show_menu();
		return 0;
	}

	test_run_item(item);
	
	return 0;
}

void test_set_unit(int unit)
{
	switch(unit)
	{
		case TEST_UNIT_FILE:
		case TEST_UNIT_FUNC:
		case TEST_UNIT_LIB:
		case TEST_UNIT_MODULE:
			test_unit = unit;
			break;

		default:
			printf("Warn: Invalid test unit!\n");
			break;
	}
}

void test_setup_menu_index(void)
{
	__test_setup_menu_index(&top_item);
}

void test_show_menu(void)
{
	int i;
	test_item_s* sub_item = NULL;
	int item_count;

	test_get_sub_item(cur_item, &sub_item, &item_count);

	for(i = 0; i < item_count; i++)
	{
		if(sub_item[i].menu.index == TEST_INVALID_INDEX)
		{
			continue;
		}

		//˵ûУΪ˵ӡ
		if(strcmp(sub_item[i].menu.name, ""))
		{
			printf("[%d] %s\n", sub_item[i].menu.index, sub_item[i].menu.name);
		}
		else
		{
			printf("[%d] %s\n", sub_item[i].menu.index, sub_item[i].name);
		}
	}
}
